import rich_click as click
from rich import box, print
from rich.table import Table

from canvas_course_tools import configfile
from canvas_course_tools.canvas_tasks import (
    CanvasTasks,
    Forbidden,
    InvalidAccessToken,
    ResourceDoesNotExist,
)
from canvas_course_tools.utils import get_canvas


@click.group()
def courses():
    """Add, remove and list Canvas courses."""
    pass


@courses.command("list")
@click.argument("server_alias", type=str, default="")
@click.option(
    "-c",
    "--codes/--no-codes",
    "use_codes",
    help="Include course codes.",
    default=False,
)
def list_courses(server_alias, use_codes):
    """List Canvas courses.

    Without an argument, list all courses previously registered with an alias.
    When called with the SERVER_ALIAS argument, list all canvas courses
    available at that server.
    """
    config = configfile.read_config()
    if server_alias:
        canvas = get_canvas(server_alias)
        try:
            courses = canvas.list_courses()
        except InvalidAccessToken:
            raise click.UsageError(f"You must update your canvas access token.")
        else:
            aliases = {
                v["course_id"]: k
                for k, v in config.get("courses", {}).items()
                if v["server"] == server_alias
            }
            for course in courses:
                course.alias = aliases.get(course.id, None)
            print_courses(courses, use_codes)
    else:
        courses = []
        if "courses" in config:
            for alias, course in config["courses"].items():
                canvas = get_canvas(course["server"])
                course = canvas.get_course(course["course_id"])
                course.alias = alias
                courses.append(course)
            print_courses(courses, use_codes)
        else:
            raise click.UsageError("No courses are registered yet.")


@courses.command("add")
@click.argument("alias")
@click.argument("server_alias")
@click.argument("course_id", type=int)
@click.option(
    "-U",
    "--update",
    is_flag=True,
    help="If alias already exists, update to the new course.",
)
def add_course(alias, course_id, server_alias, update):
    """Register a course using an alias."""
    config = configfile.read_config()
    courses = config.setdefault("courses", {})
    if alias in courses and not update:
        print(f"[bold red] Course '{alias}' already exists.[/bold red]")
    else:
        try:
            server = config["servers"][server_alias]
        except KeyError:
            print(f"[bold red] Unknown server '{server_alias}'.[/bold red]")
        else:
            canvas = CanvasTasks(server["url"], server["token"])
            try:
                canvas.get_course(course_id)
            except ResourceDoesNotExist:
                print(f"[bold red]This course ID does not exist.[/bold red]")
            except Forbidden:
                print(
                    f"[bold red]You don't have authorization for this course.[/bold red]"
                )
            else:
                courses[alias] = {"server": server_alias, "course_id": course_id}
                configfile.write_config(config)


@courses.command("remove")
@click.argument("alias", type=str)
def remove_course(alias):
    """Remove course from configuration."""
    config = configfile.read_config()
    try:
        del config["courses"][alias]
    except KeyError:
        print(f"[bold red] Unknown course '{alias}'.[/bold red]")
    else:
        configfile.write_config(config)
        print(f"Course '{alias}' removed.")


def print_courses(courses, use_codes):
    """Print a list of courses in a formatted table.

    Args:
        courses (list): list of Canvas courses
        use_codes (bool): if True, show the SIS course code
    """
    table = Table(box=box.HORIZONTALS)
    table.add_column("ID")
    table.add_column("Alias")
    if use_codes:
        table.add_column("Course Code")
    table.add_column("Name")
    table.add_column("Term")
    for course in courses:
        fields = [str(course.id), course.alias, course.name, course.term]
        if use_codes:
            fields.insert(1, course.course_code)
        table.add_row(*fields)
    print()
    print(table)



import pathlib

import appdirs
import tomli
import tomli_w


APP_NAME = "canvas-course-tools"
CONFIG_FILE = "config.toml"


def read_config():
    """Read configuration file."""
    config_path = get_config_path()
    if config_path.is_file():
        try:
            with open(config_path, "rb") as f:
                return tomli.load(f)
        except (tomli.TOMLDecodeError, UnicodeDecodeError):
            # error parsing TOML
            return {}
    else:
        return {}


def write_config(config):
    """Write configuration file.
    Args:
        config: a dictionary containing the configuration.
    """
    create_config_dir()
    config_path = get_config_path()
    # make sure that TOML conversion works before opening file
    tomli_w.dumps(config)
    with open(config_path, "wb") as f:
        # separate TOML generation from writing to file, or an exception
        # generating TOML will result in an empty file
        # correct TOML unicode handling requires writing bytes
        tomli_w.dump(config, f)


def get_config_path():
    """Get path of configuration file."""
    config_dir = pathlib.Path(appdirs.user_config_dir(APP_NAME))
    config_path = config_dir / CONFIG_FILE
    return config_path


def create_config_dir():
    """Create configuration directory if necessary."""
    config_path = get_config_path()
    config_path.parent.mkdir(parents=True, exist_ok=True)



import itertools
import os.path
import pathlib
import re
import unicodedata

from rich.console import Console

from canvas_course_tools.datatypes import GroupList, Student, StudentGroup

PARSE_STUDENT_RE = re.compile("(?P<name>.*) \((?P<id>.*)\) *(?:\[(?P<notes>.*)\])?")


def parse_group_list(text, photo_dir=None, relative_to=None):
    """Parse text and build a group list."""
    group_list = GroupList()
    current_group = StudentGroup()

    for line in text.splitlines():
        if not line:
            # empty line
            continue
        elif line.startswith("##"):
            # new group name
            group_name = line.removeprefix("##").strip()
            if current_group.students:
                # there are students in the current group, add them to the list
                group_list.groups.append(current_group)
            # create new group
            current_group = StudentGroup(name=group_name)
        elif line.startswith("#"):
            # group list name
            group_list.name = line.removeprefix("#").strip()
        else:
            # must be a student
            match = PARSE_STUDENT_RE.match(line)
            if match:
                if photo_dir:
                    photo = find_photo(match["name"], photo_dir)
                    if photo and relative_to is not None:
                        photo = pathlib.Path(os.path.relpath(photo, start=relative_to))
                else:
                    photo = None
                current_group.students.append(
                    Student(
                        name=match["name"],
                        id=match["id"],
                        notes=match["notes"],
                        photo=photo,
                    ),
                )
    group_list.groups.append(current_group)

    return group_list


def find_photo(name, photo_dir):
    """Find a photo for a given student name.

    Args:
        name (str): Name of the student
        photo_dir (pathlib.Path): Path to a directory with photos.
    """
    try:
        pattern = name + ".*"
        nfc = unicodedata.normalize("NFC", pattern)
        nfd = unicodedata.normalize("NFD", pattern)
        return next(itertools.chain(photo_dir.glob(nfc), photo_dir.glob(nfd)))
    except StopIteration:
        console = Console(stderr=True)
        console.print(f"[red]WARNING: no photo found for {name}.")
        return None



import importlib.resources
import pathlib

import jinja2
import rich_click as click
import tomli
from rich import box
from rich.console import Console
from rich.syntax import Syntax
from rich.table import Table

from canvas_course_tools.datatypes import GroupList
from canvas_course_tools.group_lists import parse_group_list

TEMPLATE_INFO_FILE = "template-info.toml"


@click.group()
def templates():
    """Generate files based on templates and group lists."""
    pass


@templates.command("list")
def list_templates():
    """List all available templates."""
    template_files = importlib.resources.files("canvas_course_tools") / "templates"
    info_file = template_files / TEMPLATE_INFO_FILE
    info = tomli.loads(info_file.read_text(encoding="utf-8"))

    table = Table(box=box.HORIZONTALS)
    table.add_column("Template")
    table.add_column("Description")
    for template in sorted(template_files.iterdir()):
        if template.is_file() and not template == info_file:
            table.add_row(template.name, info.get(template.name, ""))
    console = Console()
    console.print()
    console.print(table)


@templates.command("show")
@click.argument("template")
def show_template(template):
    """Show the contents of a template.

    The argument, TEMPLATE, should be the name of one of the templates
    registered with the app. Use the `list` command to get a list of templates.
    """
    template_files = importlib.resources.files("canvas_course_tools") / "templates"
    path = template_files / template
    console = Console()

    if not path.is_file():
        raise click.BadArgumentUsage(f"Template {template} not found!")
    elif console.is_terminal:
        syntax = Syntax.from_path(path)
        console.print()
        console.print(syntax)
    else:
        print(path.read_text(encoding="utf-8"))


@templates.command()
@click.argument("template")
@click.argument("group_list")
@click.option(
    "-f",
    "--file",
    help="Write the output to this file.",
)
@click.option(
    "-w",
    "--write",
    "auto_write",
    is_flag=True,
    help="Write the output to file based on template and group list names. This option has no effect when --file is also used.",
)
@click.option(
    "-d",
    "--dir",
    "output_dir",
    help="Write output to a file relative to this directory.",
)
@click.option(
    "-p",
    "--photos",
    "photo_dir",
    type=click.Path(
        file_okay=False, dir_okay=True, exists=True, path_type=pathlib.Path
    ),
    help="Search matching photos in this directory.",
)
def render(template, group_list, file, auto_write, output_dir, photo_dir):
    """Render a template.

    The first argument, TEMPLATE, should be the name of one of the templates
    registered with the app or a path of a template file. Use the `list` command
    to get a list of included templates.

    The second argument, GROUP_LIST, should be a path to a group list file. If a
    line starts with a #-character, the rest of the line is interpreted as a
    title. If it starts with ##, the rest of the line is interpreted as a group
    name. There can be multiple groups defined in one file. All other non-empty
    lines are interpreted as a student name with optional student id and notes
    field. It must be of the form "student name (id) [notes]". For example, the
    following is a valid group list file:



    \b
        # Physics 101
        ## Group A
        Drew Ferrell (800057) [second year]
        Amanda James (379044)
        Antonio Morris (804407) [skips thursdays]

    \b
        ## Group B
        Elizabeth Allison (312702)
        James Morales (379332)

    If the --photos option is given on the command line, for each student a
    matching photo is looked up in this directory. The name of the file should
    match the full name of the student *exactly*. E.g. "Drew Ferrel.jpg" for the
    first student in the example group list file. The extension does not matter
    and is passed as is to the template.

    \f
    Ignore the \b and \f characters in this docstring. They are to tell click to
    not wrap paragraphs (\b) and not display this note (\f).
    """
    file_contents = pathlib.Path(group_list).read_text(encoding="utf-8")

    if file or auto_write:
        output_path = build_output_path(file, output_dir, template, group_list)
        relative_to = output_path.parent
    else:
        relative_to = None

    group_list_data = parse_group_list(
        file_contents, photo_dir, relative_to=relative_to
    )
    contents = render_template(template, group_list_data)

    if not file and not auto_write:
        # output to console
        console = Console()
        if console.is_terminal:
            filetype = pathlib.Path(template).suffix.lstrip(".")
            syntax = Syntax(contents, lexer=filetype)
            console.print()
            console.print(syntax)
        else:
            print(contents)
    else:
        print(f"Writing template output to {output_path}...")
        try:
            output_path.write_text(contents, encoding="utf-8")
        except FileNotFoundError:
            raise click.BadArgumentUsage(
                f"Output file {output_path} cannot be created."
            )


def build_output_path(file, output_dir, template, group_list):
    """Build output path based on various options and arguments.

    If file is given, this will be the filename. Else a filename will be
    generated by concatenating the template name and the group_list name. The
    returned path is the filename relative to the output_dir directory.

    Args:
        file (str): the filename
        output_dir (str): the output directory
        template (str): the name of the template file
        group_list (str): the name of the group list file

    Returns:
        pathlib.Path: the generated output path
    """
    if output_dir:
        output_dir = pathlib.Path(output_dir)
    else:
        output_dir = pathlib.Path()

    if file:
        path = output_dir / file
    else:
        template_path = pathlib.Path(template)
        filename = template_path.stem + "-" + pathlib.Path(group_list).stem
        filepath = pathlib.Path(filename).with_suffix(template_path.suffix)
        path = output_dir / filepath
    return path


def render_template(template_name, group_list: GroupList):
    """Render a template.

    Args:
        template_name (str): the name of the template or a path to a template
            file.
        group_list (GroupList): the group list as input for the template
    """
    if pathlib.Path(template_name).is_file():
        env = jinja2.Environment(loader=jinja2.FileSystemLoader("."))
    else:
        env = jinja2.Environment(
            loader=jinja2.PackageLoader("canvas_course_tools", "templates"),
        )
    try:
        template = env.get_template(template_name, globals={"zip": zip})
    except jinja2.exceptions.TemplateNotFound:
        raise click.BadArgumentUsage(f"Template {template_name} not found!")
    return template.render(title=group_list.name, groups=group_list.groups)



import pathlib

import rich_click as click
from rich import print
from rich.progress import Progress

from canvas_course_tools.canvas_tasks import (
    CanvasObjectExistsError,
    Forbidden,
    ResourceDoesNotExist,
)
from canvas_course_tools.group_lists import parse_group_list
from canvas_course_tools.utils import find_course


@click.group()
def groups():
    """Create Canvas groups based on group lists."""
    pass


@groups.command("create")
@click.argument("course_alias")
@click.argument("group_list")
@click.option(
    "--overwrite/--no-overwrite",
    default=False,
    help="Whether or not to overwrite an existing groupset.",
)
def create_canvas_groups(course_alias, group_list, overwrite):
    """Create canvas groups.

    Create canvas groups in course COURSE_ALIAS based on the group list defined
    in GROUP_LIST. The groups will be placed in GroupSet named after the title
    in the group list file.

    The GROUP_LIST argument should be a path to a group list file. If a line
    starts with a #-character, the rest of the line is interpreted as a title.
    If it starts with ##, the rest of the line is interpreted as a group name.
    There can be multiple groups defined in one file. All other non-empty lines
    are interpreted as a student name with optional student id and notes field.
    It must be of the form "student name (id) [notes]". For example, the
    following is a valid group list file:



    \b
        # Physics 101
        ## Group A
        Drew Ferrell (800057) [second year]
        Amanda James (379044)
        Antonio Morris (804407) [skips thursdays]

    \b
        ## Group B
        Elizabeth Allison (312702)
        James Morales (379332)

    \f
    Ignore the \b and \f characters in this docstring. They are to tell click to
    not wrap paragraphs (\b) and not display this note (\f).
    """
    canvas, course = find_course(course_alias)

    file_contents = pathlib.Path(group_list).read_text(encoding="utf-8")
    group_list = parse_group_list(file_contents)

    print(f"Creating GroupSet {group_list.name}...")
    try:
        groupset = canvas.create_groupset(group_list.name, course, overwrite)
    except CanvasObjectExistsError:
        raise click.BadArgumentUsage(
            f"Canvas groupset '{group_list.name}' already exists. You can use --overwrite."
        )

    with Progress() as progress:
        task_groups = progress.add_task(
            description="Creating groups...", total=len(group_list.groups)
        )
        task_students = progress.add_task(description="Adding students to group...")

        for group in group_list.groups:
            canvas_group = canvas.create_group(group.name, groupset)
            progress.reset(task_students, total=len(group.students))
            for student in group.students:
                try:
                    canvas.add_student_to_group(student, canvas_group)
                except ResourceDoesNotExist:
                    print(f"[red]WARNING: student {student.name} does not exist.")
                except Forbidden:
                    print(
                        f"[red]WARNING: you do not have authorization to add student {student.name}."
                    )
                progress.advance(task_students)
            progress.advance(task_groups)
            print(f"Created {group.name}.")

    print("Done")


@groups.command("list")
@click.argument("course_alias")
def list_groups(course_alias: str) -> None:
    """List all groups and groupsets in a course.

    List all groups in course COURSE_ALIAS, for all group sets (a.k.a. group
    categories). Each group set also lists the group set ID in brackets. This
    can be useful if you want to list all students in a group set. See also the
    `canvas students` command group.
    """
    canvas, course = find_course(course_alias)
    for group_set in canvas.list_groupsets(course):
        print(f"{group_set.name} ({group_set.id}):")
        for group in canvas.list_groups(group_set):
            print(f"\t{group.name}")
        print()



import click

from canvas_course_tools import configfile
from canvas_course_tools.canvas_tasks import CanvasTasks, Course


def get_canvas(server_alias):
    config = configfile.read_config()
    try:
        server = config["servers"][server_alias]
    except KeyError:
        raise click.UsageError(f"Unknown server '{server_alias}'.")
    else:
        return CanvasTasks(server["url"], server["token"])


def find_course(course_alias: str) -> tuple[CanvasTasks, Course]:
    config = configfile.read_config()
    try:
        server, course_id = (
            config["courses"][course_alias][k] for k in ("server", "course_id")
        )
    except KeyError:
        raise click.BadArgumentUsage(f"Unknown course {course_alias}.")
    canvas = get_canvas(server)
    course = canvas.get_course(course_id)
    return canvas, course



import pathlib
from dataclasses import dataclass, field

import canvasapi


@dataclass
class Course:
    id: int
    name: str
    course_code: str
    term: str
    alias: str | None = None
    _course: canvasapi.course.Course | None = field(default=None, repr=False)


@dataclass
class GroupSet:
    id: int
    name: str
    _group_set: canvasapi.group.GroupCategory | None = field(default=None, repr=False)


@dataclass
class Group:
    id: int
    name: str
    _group: canvasapi.group.Group | None = field(default=None, repr=False)


@dataclass
class Student:
    id: int
    name: str
    sortable_name: str | None = None
    first_name: str = field(init=False)
    last_name: str = field(init=False)
    notes: str | None = None
    photo: pathlib.Path | None = None

    def __post_init__(self):
        # Parse human names, allowing for underscores to group parts of the name
        self.first_name, _, self.last_name = self.name.partition(" ")
        self.first_name = self.first_name.replace("_", " ")
        self.name = self.name.replace("_", " ")


@dataclass
class Section:
    id: int
    name: str
    students: list[Student]


@dataclass
class StudentGroup:
    name: str
    students: list[Student] = field(default_factory=list)


@dataclass
class GroupList:
    name: str
    groups: list[StudentGroup] = field(default_factory=list)


@dataclass
class AssignmentGroup:
    id: int
    name: str
    course: Course


@dataclass
class Assignment:
    id: int
    name: str
    _api: canvasapi.assignment.Assignment | None = field(default=None, repr=False)


@dataclass
class Attachment:
    name: str
    url: str
    content_type: str


@dataclass
class Submission:
    student: Student
    # submission_comments: list[str]
    attachments: list[Attachment]



import rich_click as click
from rich import box, print
from rich.table import Table

from canvas_course_tools import configfile


@click.group()
def servers():
    """Add, remove and list Canvas servers."""
    pass


@servers.command("list")
def show_servers():
    """List the registered servers."""
    config = configfile.read_config()
    try:
        servers = config["servers"]
    except KeyError:
        pass
    else:
        table = Table(box=box.HORIZONTALS)
        table.add_column("Alias")
        table.add_column("URL")
        for alias, settings in servers.items():
            table.add_row(alias, settings["url"])
        print()
        print(table)


@servers.command("add")
@click.argument("alias", type=str)
@click.argument("url", type=str)
@click.argument("token", type=str)
@click.option(
    "-f",
    "--force",
    is_flag=True,
    help="If alias already exists, force overwrite.",
)
def add_server(alias, url, token, force):
    """Register an alias for a server with corresponding access token.

    Example:

        canvas servers add school http://school.example.com/ 123~secret
    """
    config = configfile.read_config()
    servers = config.setdefault("servers", {})
    if alias in servers and not force:
        print(f"[bold red] Server '{alias}' already exists.[/bold red]")
    else:
        servers[alias] = {"url": url, "token": token}
        configfile.write_config(config)


@servers.command("remove")
@click.argument("alias", type=str)
def remove_server(alias):
    """Remove server from configuration."""
    config = configfile.read_config()
    try:
        del config["servers"][alias]
    except KeyError:
        print(f"[bold red] Unknown server '{alias}'.[/bold red]")
    else:
        configfile.write_config(config)
        print(f"Server '{alias}' removed.")



"""Canvas course tools command-line application.

This command-line application enables you to quickly perform a variety of tasks
that would otherwise require you to log in to a Canvas site. Since using this
tool requires no mouse-clicks at all, these tasks are performed much quicker and
easier than through the web interface.
"""

import rich_click as click
import trogon

from canvas_course_tools import __version__
from canvas_course_tools.courses import courses
from canvas_course_tools.groups import groups
from canvas_course_tools.servers import servers
from canvas_course_tools.students import students
from canvas_course_tools.templates import templates


@trogon.tui()
@click.group()
@click.version_option(version=__version__)
def cli():
    pass


cli.add_command(servers)
cli.add_command(courses)
cli.add_command(students)
cli.add_command(templates)
cli.add_command(groups)


if __name__ == "__main__":
    cli()



import canvasapi
from canvasapi import Canvas
from canvasapi.exceptions import Forbidden, InvalidAccessToken, ResourceDoesNotExist

from canvas_course_tools.datatypes import (
    Assignment,
    AssignmentGroup,
    Attachment,
    Course,
    Group,
    GroupSet,
    Section,
    Student,
    Submission,
)


class CanvasObjectExistsError(Exception):
    pass


class CanvasTasks:

    """Collection of canvas-related tasks."""

    def __init__(self, url, token):
        """Initialize the class.

        Args:
            url: a string containing the Canvas server URL
            token: a string containing the Canvas API access token
        """
        self.canvas = Canvas(url, token)

    def list_courses(self):
        """List Canvas courses.

        Returns:
            A list of Canvas course objects.
        """
        courses = self.canvas.get_courses(include=["term"])
        return [create_course_object(course) for course in courses]

    def get_course(self, course_id):
        """Get a Canvas course by id.

        Args:
            course_id: a Canvas course id.

        Returns:
            A Canvas course object.
        """
        course = self.canvas.get_course(course_id, include=["term"])
        return create_course_object(course)

    def get_students(self, course_id):
        """Get all students in a course.

        Args:
            course_id (integer): the Canvas course id

        Returns:
            list: a list of student objects
        """
        course = self.canvas.get_course(course_id)
        students = course.get_users(enrollment_type=["student"])
        return [create_student_object(student) for student in students]

    def get_sections(self, course_id):
        """Get a list of sections, including students

        Args:
            course_id (int): the course id

        Returns:
            List[Section]: A list of Sections
        """
        course = self.canvas.get_course(course_id)
        sections = course.get_sections(include=["students"])
        return [
            Section(
                id=section.id,
                name=section.name,
                students=[
                    Student(
                        id=student["id"],
                        name=student["short_name"],
                        sortable_name=student["sortable_name"],
                    )
                    for student in section.students
                ],
            )
            for section in sections
            if section.students is not None
        ]

    def create_groupset(self, name, course, overwrite):
        """Create groupset in course.

        Args:
            name (string): name of the groupset
            course (Course): the course in which to create the groupset
            overwrite (bool): whether or not to overwrite an existing groupset

        Returns:
            GroupSet: the newly created groupset
        """
        canvas_course = self.canvas.get_course(course.id)
        groupsets = list(canvas_course.get_group_categories())
        groupset_names = [g.name for g in groupsets]
        if name in groupset_names:
            if not overwrite:
                raise CanvasObjectExistsError("Groupset already exists.")
            else:
                for groupset in [g for g in groupsets if g.name == name]:
                    # we don't expect groupset with the same name, but delete them all in any case
                    groupset.delete()
        groupset = canvas_course.create_group_category(name)
        return GroupSet(id=groupset.id, name=name)

    def list_groupsets(self, course: Course) -> list[GroupSet]:
        """List groupsets in a course.

        Args:
            course (Course): the course containing the groupsets.

        Returns:
            list[GroupSet]: a list of GroupSet objects.
        """
        groupsets = course._course.get_group_categories()
        return [
            GroupSet(id=groupset.id, name=groupset.name, _group_set=groupset)
            for groupset in groupsets
        ]

    def get_groupset(self, group_set_id: int) -> GroupSet:
        """Get a groupset by id

        Args:
            group_set_id (int): the groupset ID.

        Returns:
            GroupSet: the requests groupset object.
        """
        groupset = self.canvas.get_group_category(group_set_id)
        return GroupSet(id=groupset.id, name=groupset.name, _group_set=groupset)

    def create_group(self, group_name, group_set):
        """Create a group inside a GroupSet.

        Args:
            group_name (str): name of the group to create.
            group_set (GroupSet): the GroupSet in which the group will be
                created.

        Returns:
            Group: the newly created group.
        """
        groupset = self.canvas.get_group_category(group_set.id)
        group = groupset.create_group(name=group_name)
        return Group(id=group.id, name=group_name)

    def list_groups(self, group_set: GroupSet) -> list[Group]:
        groups = group_set._group_set.get_groups()
        return [Group(id=group.id, name=group.name, _group=group) for group in groups]

    def add_student_to_group(self, student, group):
        """Add student to a group.

        Args:
            student (Student): the student to add to the group.
            group (Group): the group in which to place the student.
        """
        canvas_group = self.canvas.get_group(group.id)
        canvas_group.create_membership(student.id)

    def get_students_in_group(self, group: Group) -> list[Student]:
        students = group._group.get_users()
        return [create_student_object(student) for student in students]

    def get_assignment_groups(self, course: Course) -> list[AssignmentGroup]:
        groups = course._course.get_assignment_groups()
        return [
            AssignmentGroup(id=group.id, name=group.name, course=course)
            for group in groups
        ]

    def get_assignments_for_group(self, group: AssignmentGroup) -> list[Assignment]:
        assignments = group.course._course.get_assignments_for_group(group.id)
        return [
            Assignment(id=assignment.id, name=assignment.name, _api=assignment)
            for assignment in assignments
        ]

    def get_submission(self, assignment: Assignment, student: Student) -> Submission:
        submission = assignment._api.get_submission(user=student.id)
        attachments = [
            Attachment(
                name=a.filename, url=a.url, content_type=getattr(a, "content-type")
            )
            for a in submission.attachments
        ]
        return Submission(student=student, attachments=attachments)


def create_course_object(course: canvasapi.course.Course):
    return Course(
        id=course.id,
        name=course.name,
        course_code=course.course_code,
        term=course.term["name"],
        _course=course,
    )


def create_student_object(student: canvasapi.user.User):
    return Student(
        id=student.id,
        name=student.short_name,
        sortable_name=student.sortable_name,
    )
